@@ -4,6 +4,7 @@ use crate::{
     arithmetic::Exponent, decorator::Decorator, markup::Markup, number::Number, prefix::Prefix,
     pretty_print::PrettyPrint, resolver::ModulePath,
 };
+use itertools::Itertools;
 use num_traits::Signed;
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -76,7 +77,7 @@ pub enum Expression {
         rhs: Box<Expression>,
         span_op: Option<Span>, // not available for implicit multiplication and unicode exponents
     },
-    FunctionCall(Span, Span, String, Vec<Expression>),
+    FunctionCall(Span, Span, Box<Expression>, Vec<Expression>),
     Boolean(Span, bool),
     String(Span, Vec<StringPart>),
     Condition(Span, Box<Expression>, Box<Expression>, Box<Expression>),
@@ -215,6 +216,8 @@ pub enum TypeAnnotation {
     DimensionExpression(DimensionExpression),
     Bool(Span),
     String(Span),
+    DateTime(Span),
+    Fn(Span, Vec<TypeAnnotation>, Box<TypeAnnotation>),
 }
 
 impl TypeAnnotation {
@@ -223,6 +226,8 @@ impl TypeAnnotation {
             TypeAnnotation::DimensionExpression(d) => d.full_span(),
             TypeAnnotation::Bool(span) => *span,
             TypeAnnotation::String(span) => *span,
+            TypeAnnotation::DateTime(span) => *span,
+            TypeAnnotation::Fn(span, _, _) => *span,
         }
     }
 }
@@ -233,6 +238,22 @@ impl PrettyPrint for TypeAnnotation {
             TypeAnnotation::DimensionExpression(d) => d.pretty_print(),
             TypeAnnotation::Bool(_) => m::type_identifier("Bool"),
             TypeAnnotation::String(_) => m::type_identifier("String"),
+            TypeAnnotation::DateTime(_) => m::type_identifier("DateTime"),
+            TypeAnnotation::Fn(_, parameter_types, return_type) => {
+                m::type_identifier("Fn")
+                    + m::operator("[(")
+                    + Itertools::intersperse(
+                        parameter_types.iter().map(|t| t.pretty_print()),
+                        m::operator(",") + m::space(),
+                    )
+                    .sum()
+                    + m::operator(")")
+                    + m::space()
+                    + m::operator("->")
+                    + m::space()
+                    + return_type.pretty_print()
+                    + m::operator("]")
+            }
         }
     }
 }
@@ -364,6 +385,10 @@ impl ReplaceSpans for TypeAnnotation {
             }
             TypeAnnotation::Bool(_) => TypeAnnotation::Bool(Span::dummy()),
             TypeAnnotation::String(_) => TypeAnnotation::String(Span::dummy()),
+            TypeAnnotation::DateTime(_) => TypeAnnotation::DateTime(Span::dummy()),
+            TypeAnnotation::Fn(_, pt, rt) => {
+                TypeAnnotation::Fn(Span::dummy(), pt.clone(), rt.clone())
+            }
         }
     }
 }
@@ -437,10 +462,10 @@ impl ReplaceSpans for Expression {
                 rhs: Box::new(rhs.replace_spans()),
                 span_op: Some(Span::dummy()),
             },
-            Expression::FunctionCall(_, _, name, args) => Expression::FunctionCall(
+            Expression::FunctionCall(_, _, callable, args) => Expression::FunctionCall(
                 Span::dummy(),
                 Span::dummy(),
-                name.clone(),
+                Box::new(callable.replace_spans()),
                 args.iter().map(|a| a.replace_spans()).collect(),
             ),
             Expression::Boolean(_, val) => Expression::Boolean(Span::dummy(), *val),
