@@ -7,10 +7,12 @@ use crate::interpreter::{
 };
 use crate::name_resolution::LAST_RESULT_IDENTIFIERS;
 use crate::prefix::Prefix;
+use crate::prefix_parser::AcceptsPrefix;
 use crate::pretty_print::PrettyPrint;
 use crate::typed_ast::{BinaryOperator, Expression, Statement, StringPart, UnaryOperator};
-use crate::unit::Unit;
+use crate::unit::{CanonicalName, Unit};
 use crate::unit_registry::{UnitMetadata, UnitRegistry};
+use crate::value::FunctionReference;
 use crate::vm::{Constant, ExecutionContext, Op, Vm};
 use crate::{decorator, ffi};
 
@@ -35,6 +37,8 @@ pub struct BytecodeInterpreter {
     locals: Vec<Vec<Local>>,
     // Maps names of units to indices of the respective constants in the VM
     unit_name_to_constant_index: HashMap<String, u16>,
+    /// List of functions
+    functions: HashMap<String, bool>,
 }
 
 impl BytecodeInterpreter {
@@ -45,7 +49,7 @@ impl BytecodeInterpreter {
                 self.vm.add_op1(Op::LoadConstant, index);
             }
             Expression::Identifier(_span, identifier, _type) => {
-                // Searching in reverse order ensures that we find the innermost identifer of that name first (shadowing)
+                // Searching in reverse order ensures that we find the innermost identifier of that name first (shadowing)
 
                 let current_depth = self.locals.len() - 1;
 
@@ -61,8 +65,17 @@ impl BytecodeInterpreter {
                     self.vm.add_op1(Op::GetUpvalue, upvalue_position as u16);
                 } else if LAST_RESULT_IDENTIFIERS.contains(&identifier.as_str()) {
                     self.vm.add_op(Op::GetLastResult);
+                } else if let Some(is_foreign) = self.functions.get(identifier) {
+                    let index = self
+                        .vm
+                        .add_constant(Constant::FunctionReference(if *is_foreign {
+                            FunctionReference::Foreign(identifier.clone())
+                        } else {
+                            FunctionReference::Normal(identifier.clone())
+                        }));
+                    self.vm.add_op1(Op::LoadConstant, index);
                 } else {
-                    unreachable!("Unknown identifier {identifier}")
+                    unreachable!("Unknown identifier '{identifier}'")
                 }
             }
             Expression::UnitIdentifier(_span, prefix, unit_name, _full_name, _type) => {
@@ -112,10 +125,35 @@ impl BytecodeInterpreter {
                 };
                 self.vm.add_op(op);
             }
+            Expression::BinaryOperatorForDate(_span, operator, lhs, rhs, type_) => {
+                self.compile_expression(lhs)?;
+                self.compile_expression(rhs)?;
+
+                // if the result is a duration:
+                let op = if type_.is_dtype() {
+                    // the VM will need to return a value with the units of Seconds.  so look up that unit here, and push it
+                    // onto the stack, so the VM can easily reference it.
+                    // TODO: We do not want to hard-code 'second' here. Instead, we might
+                    // introduce a decorator to register the 'second' unit in the prelude for
+                    // this specific purpose. We also need to handle errors in case no such unit
+                    // was registered.
+                    let second_idx = self.unit_name_to_constant_index.get("second");
+                    self.vm.add_op1(Op::LoadConstant, *second_idx.unwrap());
+                    Op::DiffDateTime
+                } else {
+                    match operator {
+                        BinaryOperator::Add => Op::AddToDateTime,
+                        BinaryOperator::Sub => Op::SubFromDateTime,
+                        _ => unreachable!("{operator:?} is not valid with a DateTime"), // should be unreachable, because the typechecker will error first
+                    }
+                };
+
+                self.vm.add_op(op);
+            }
             Expression::FunctionCall(_span, _full_span, name, args, _type) => {
                 // Put all arguments on top of the stack
                 for arg in args {
-                    self.compile_expression(arg)?;
+                    self.compile_expression_with_simplify(arg)?;
                 }
 
                 if let Some(idx) = self.vm.get_ffi_callable_idx(name) {
@@ -127,6 +165,17 @@ impl BytecodeInterpreter {
                     self.vm.add_op2(Op::Call, idx, args.len() as u16); // TODO: check overflow
                 }
             }
+            Expression::CallableCall(_span, callable, args, _type) => {
+                // Put all arguments on top of the stack
+                for arg in args {
+                    self.compile_expression_with_simplify(arg)?;
+                }
+
+                // Put the callable on top of the stack
+                self.compile_expression(callable)?;
+
+                self.vm.add_op1(Op::CallCallable, args.len() as u16);
+            }
             Expression::Boolean(_, val) => {
                 let index = self.vm.add_constant(Constant::Boolean(*val));
                 self.vm.add_op1(Op::LoadConstant, index);
@@ -180,12 +229,13 @@ impl BytecodeInterpreter {
             | Expression::Identifier(..)
             | Expression::UnitIdentifier(..)
             | Expression::FunctionCall(..)
+            | Expression::CallableCall(..)
             | Expression::UnaryOperator(..)
             | Expression::BinaryOperator(_, BinaryOperator::ConvertTo, _, _, _)
             | Expression::Boolean(..)
             | Expression::String(..)
             | Expression::Condition(..) => {}
-            Expression::BinaryOperator(..) => {
+            Expression::BinaryOperator(..) | Expression::BinaryOperatorForDate(..) => {
                 self.vm.add_op(Op::FullSimplify);
             }
         }
@@ -203,13 +253,13 @@ impl BytecodeInterpreter {
                 let current_depth = self.current_depth();
 
                 // For variables, we ignore the prefix info and only use the names
-                let aliases = crate::decorator::name_and_aliases(&identifier, &decorators)
+                let aliases = crate::decorator::name_and_aliases(identifier, decorators)
                     .map(|(name, _)| name)
                     .cloned()
                     .collect::<Vec<_>>();
                 let metadata = LocalMetadata {
-                    name: crate::decorator::name(&decorators),
-                    url: crate::decorator::url(&decorators),
+                    name: crate::decorator::name(decorators),
+                    url: crate::decorator::url(decorators),
                     aliases: aliases.clone(),
                 };
 
@@ -250,6 +300,8 @@ impl BytecodeInterpreter {
                 self.locals.pop();
 
                 self.vm.end_function();
+
+                self.functions.insert(name.clone(), false);
             }
             Statement::DefineFunction(
                 name,
@@ -272,6 +324,8 @@ impl BytecodeInterpreter {
                         parameters.len()..=parameters.len()
                     },
                 );
+
+                self.functions.insert(name.clone(), true);
             }
             Statement::DefineDimension(_name, _dexprs) => {
                 // Declaring a dimension is like introducing a new type. The information
@@ -291,6 +345,9 @@ impl BytecodeInterpreter {
                             readable_type: readable_type.clone(),
                             aliases,
                             name: decorator::name(decorators),
+                            canonical_name: decorator::get_canonical_unit_name(
+                                unit_name, decorators,
+                            ),
                             url: decorator::url(decorators),
                             binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                             metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
@@ -300,7 +357,7 @@ impl BytecodeInterpreter {
 
                 let constant_idx = self.vm.add_constant(Constant::Unit(Unit::new_base(
                     unit_name,
-                    &crate::decorator::get_canonical_unit_name(unit_name.as_str(), &decorators[..]),
+                    crate::decorator::get_canonical_unit_name(unit_name.as_str(), &decorators[..]),
                 )));
                 for (name, _) in decorator::name_and_aliases(unit_name, decorators) {
                     self.unit_name_to_constant_index
@@ -312,20 +369,28 @@ impl BytecodeInterpreter {
                     .map(|(name, ap)| (name.clone(), ap))
                     .collect();
 
-                let constant_idx = self
-                    .vm
-                    .add_constant(Constant::Unit(Unit::new_base("<dummy>", "<dummy>"))); // TODO: dummy is just a temp. value until the SetUnitConstant op runs
+                let constant_idx = self.vm.add_constant(Constant::Unit(Unit::new_base(
+                    "<dummy>",
+                    CanonicalName {
+                        name: "<dummy>".to_string(),
+                        accepts_prefix: AcceptsPrefix::both(),
+                    },
+                ))); // TODO: dummy is just a temp. value until the SetUnitConstant op runs
                 let unit_information_idx = self.vm.add_unit_information(
                     unit_name,
-                    Some(&crate::decorator::get_canonical_unit_name(
-                        unit_name.as_str(),
-                        &decorators[..],
-                    )),
+                    Some(
+                        &crate::decorator::get_canonical_unit_name(
+                            unit_name.as_str(),
+                            &decorators[..],
+                        )
+                        .name,
+                    ),
                     UnitMetadata {
                         type_: type_.clone(),
                         readable_type: readable_type.clone(),
                         aliases,
                         name: decorator::name(decorators),
+                        canonical_name: decorator::get_canonical_unit_name(unit_name, decorators),
                         url: decorator::url(decorators),
                         binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                         metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
@@ -412,6 +477,7 @@ impl Interpreter for BytecodeInterpreter {
             vm: Vm::new(),
             locals: vec![vec![]],
             unit_name_to_constant_index: HashMap::new(),
+            functions: HashMap::new(),
         }
     }
 
