@@ -1,14 +1,19 @@
 mod arithmetic;
 mod ast;
+#[cfg(feature = "html-formatter")]
+pub mod buffered_writer;
 mod bytecode_interpreter;
 mod column_formatter;
 mod currency;
+mod datetime;
 mod decorator;
 pub mod diagnostic;
 mod dimension;
 mod ffi;
 mod gamma;
 pub mod help;
+#[cfg(feature = "html-formatter")]
+pub mod html_formatter;
 mod interpreter;
 pub mod keywords;
 pub mod markup;
@@ -128,12 +133,20 @@ impl Context {
         ExchangeRatesCache::set_from_xml(xml_content);
     }
 
-    pub fn variable_names(&self) -> &[String] {
-        &self.prefix_transformer.variable_names
+    pub fn variable_names(&self) -> impl Iterator<Item = String> + '_ {
+        self.prefix_transformer
+            .variable_names
+            .iter()
+            .filter(|name| !name.starts_with('_'))
+            .cloned()
     }
 
-    pub fn function_names(&self) -> &[String] {
-        &self.prefix_transformer.function_names
+    pub fn function_names(&self) -> impl Iterator<Item = String> + '_ {
+        self.prefix_transformer
+            .function_names
+            .iter()
+            .filter(|name| !name.starts_with('_'))
+            .cloned()
     }
 
     pub fn unit_names(&self) -> &[Vec<String>] {
@@ -145,13 +158,13 @@ impl Context {
     }
 
     pub fn print_environment(&self) -> Markup {
-        let mut functions = Vec::from(self.function_names());
+        let mut functions: Vec<_> = self.function_names().collect();
         functions.sort();
         let mut dimensions = Vec::from(self.dimension_names());
         dimensions.sort();
         let mut units = Vec::from(self.unit_names());
         units.sort();
-        let mut variables = Vec::from(self.variable_names());
+        let mut variables: Vec<_> = self.variable_names().collect();
         variables.sort();
 
         let mut output = m::empty();
@@ -179,7 +192,7 @@ impl Context {
     }
 
     pub fn print_functions(&self) -> Markup {
-        self.print_sorted(self.function_names().into(), FormatType::Identifier)
+        self.print_sorted(self.function_names().collect(), FormatType::Identifier)
     }
 
     pub fn print_dimensions(&self) -> Markup {
@@ -187,7 +200,7 @@ impl Context {
     }
 
     pub fn print_variables(&self) -> Markup {
-        self.print_sorted(self.variable_names().into(), FormatType::Identifier)
+        self.print_sorted(self.variable_names().collect(), FormatType::Identifier)
     }
 
     pub fn print_units(&self) -> Markup {
@@ -232,7 +245,7 @@ impl Context {
                 if add_paren {
                     words.push(format!("{}(", function));
                 } else {
-                    words.push(format!("{}", function));
+                    words.push(function.to_string());
                 }
             }
 
@@ -375,7 +388,7 @@ impl Context {
             }
 
             if let Ok((_, results)) = self.interpret(keyword, CodeSource::Internal) {
-                help += m::nl() + results.to_markup(None, self.dimension_registry(), true);
+                help += m::nl() + results.to_markup(None, self.dimension_registry(), true, true);
             }
 
             return help;
