@@ -30,7 +30,7 @@ pub enum TokenizerErrorKind {
     #[error("Unterminated string")]
     UnterminatedString,
 
-    #[error("Unterminated {{...}} interpolation in this string")]
+    #[error("Unterminated string interpolation")]
     UnterminatedStringInterpolation,
 
     #[error("Unexpected '{{' inside string interpolation")]
@@ -51,6 +51,8 @@ pub enum TokenKind {
     // Brackets
     LeftParen,
     RightParen,
+    LeftBracket,
+    RightBracket,
 
     // Operators and special signs
     Plus,
@@ -80,7 +82,7 @@ pub enum TokenKind {
 
     // Keywords
     Let,
-    Fn,
+    Fn, // 'fn'
     Dimension,
     Unit,
     Use,
@@ -95,6 +97,9 @@ pub enum TokenKind {
     Else,
 
     String,
+    DateTime,
+
+    CapitalFn, // 'Fn'
 
     Long,
     Short,
@@ -353,6 +358,8 @@ impl Tokenizer {
             m.insert("then", TokenKind::Then);
             m.insert("else", TokenKind::Else);
             m.insert("String", TokenKind::String);
+            m.insert("DateTime", TokenKind::DateTime);
+            m.insert("Fn", TokenKind::CapitalFn);
             // Keep this list in sync with keywords::KEYWORDS!
             m
         });
@@ -383,6 +390,8 @@ impl Tokenizer {
         let kind = match current_char {
             '(' => TokenKind::LeftParen,
             ')' => TokenKind::RightParen,
+            '[' => TokenKind::LeftBracket,
+            ']' => TokenKind::RightBracket,
             '≤' => TokenKind::LessOrEqual,
             '<' if self.match_char('=') => TokenKind::LessOrEqual,
             '<' => TokenKind::LessThan,
@@ -532,8 +541,8 @@ impl Tokenizer {
                     return Err(TokenizerError {
                         kind: TokenizerErrorKind::UnterminatedStringInterpolation,
                         span: Span {
-                            start: self.string_start,
-                            end: self.current,
+                            start: self.interpolation_start,
+                            end: self.last,
                             code_source_id: self.code_source_id,
                         },
                     });
