@@ -1,15 +1,15 @@
 use itertools::Itertools;
 
-use crate::arithmetic::Exponent;
+use crate::arithmetic::{Exponent, Rational};
 use crate::ast::ProcedureKind;
 pub use crate::ast::{BinaryOperator, DimensionExpression, UnaryOperator};
 use crate::dimension::DimensionRegistry;
-use crate::markup as m;
 use crate::{
     decorator::Decorator, markup::Markup, number::Number, prefix::Prefix,
     prefix_parser::AcceptsPrefix, pretty_print::PrettyPrint, registry::BaseRepresentation,
     span::Span,
 };
+use crate::{markup as m, BaseRepresentationFactor};
 
 /// Dimension type
 pub type DType = BaseRepresentation;
@@ -40,6 +40,16 @@ impl DType {
             }
         }
     }
+    /// Is the current dimension type the Time dimension?
+    ///
+    /// This is special helper that's useful when dealing with DateTimes
+    pub fn is_time_dimension(&self) -> bool {
+        *self
+            == BaseRepresentation::from_factor(BaseRepresentationFactor(
+                "Time".into(),
+                Rational::from_integer(1),
+            ))
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -47,6 +57,8 @@ pub enum Type {
     Dimension(DType),
     Boolean,
     String,
+    DateTime,
+    Fn(Vec<Type>, Box<Type>),
 }
 
 impl std::fmt::Display for Type {
@@ -55,6 +67,14 @@ impl std::fmt::Display for Type {
             Type::Dimension(d) => d.fmt(f),
             Type::Boolean => write!(f, "Bool"),
             Type::String => write!(f, "String"),
+            Type::DateTime => write!(f, "DateTime"),
+            Type::Fn(param_types, return_type) => {
+                write!(
+                    f,
+                    "Fn[({ps}) -> {return_type}]",
+                    ps = param_types.iter().map(|p| p.to_string()).join(", ")
+                )
+            }
         }
     }
 }
@@ -65,6 +85,22 @@ impl PrettyPrint for Type {
             Type::Dimension(d) => d.pretty_print(),
             Type::Boolean => m::keyword("Bool"),
             Type::String => m::keyword("String"),
+            Type::DateTime => m::keyword("DateTime"),
+            Type::Fn(param_types, return_type) => {
+                m::type_identifier("Fn")
+                    + m::operator("[(")
+                    + Itertools::intersperse(
+                        param_types.iter().map(|t| t.pretty_print()),
+                        m::operator(",") + m::space(),
+                    )
+                    .sum()
+                    + m::operator(")")
+                    + m::space()
+                    + m::operator("->")
+                    + m::space()
+                    + return_type.pretty_print()
+                    + m::operator("]")
+            }
         }
     }
 }
@@ -84,6 +120,10 @@ impl Type {
     pub fn is_dtype(&self) -> bool {
         matches!(self, Type::Dimension(..))
     }
+
+    pub fn is_fn_type(&self) -> bool {
+        matches!(self, Type::Fn(..))
+    }
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -122,7 +162,20 @@ pub enum Expression {
         Box<Expression>,
         Type,
     ),
+    /// A special binary operator that has a DateTime as one (or both) of the operands
+    BinaryOperatorForDate(
+        Option<Span>,
+        BinaryOperator,
+        /// LHS must evaluate to a DateTime
+        Box<Expression>,
+        /// RHS can evaluate to a DateTime or a quantity of type Time
+        Box<Expression>,
+        Type,
+    ),
+    // A 'proper' function call
     FunctionCall(Span, Span, String, Vec<Expression>, Type),
+    // A call via a function object
+    CallableCall(Span, Box<Expression>, Vec<Expression>, Type),
     Boolean(Span, bool),
     Condition(Span, Box<Expression>, Box<Expression>, Box<Expression>),
     String(Span, Vec<StringPart>),
@@ -142,7 +195,15 @@ impl Expression {
                 }
                 span
             }
+            Expression::BinaryOperatorForDate(span_op, _op, lhs, rhs, ..) => {
+                let mut span = lhs.full_span().extend(&rhs.full_span());
+                if let Some(span_op) = span_op {
+                    span = span.extend(span_op);
+                }
+                span
+            }
             Expression::FunctionCall(_identifier_span, full_span, _, _, _) => *full_span,
+            Expression::CallableCall(full_span, _, _, _) => *full_span,
             Expression::Boolean(span, _) => *span,
             Expression::Condition(span_if, _, _, then_expr) => {
                 span_if.extend(&then_expr.full_span())
@@ -195,7 +256,9 @@ impl Expression {
             Expression::UnitIdentifier(_, _, _, _, _type) => _type.clone(),
             Expression::UnaryOperator(_, _, _, type_) => type_.clone(),
             Expression::BinaryOperator(_, _, _, _, type_) => type_.clone(),
+            Expression::BinaryOperatorForDate(_, _, _, _, type_, ..) => type_.clone(),
             Expression::FunctionCall(_, _, _, _, type_) => type_.clone(),
+            Expression::CallableCall(_, _, _, type_) => type_.clone(),
             Expression::Boolean(_, _) => Type::Boolean,
             Expression::Condition(_, _, then, _) => then.get_type(),
             Expression::String(_, _) => Type::String,
@@ -390,7 +453,7 @@ impl PrettyPrint for Statement {
 }
 
 fn pretty_scalar(n: Number) -> Markup {
-    m::value(format!("{}", n.pretty_print()))
+    m::value(n.pretty_print())
 }
 
 fn with_parens(expr: &Expression) -> Markup {
@@ -399,10 +462,12 @@ fn with_parens(expr: &Expression) -> Markup {
         | Expression::Identifier(..)
         | Expression::UnitIdentifier(..)
         | Expression::FunctionCall(..)
+        | Expression::CallableCall(..)
         | Expression::Boolean(..)
         | Expression::String(..) => expr.pretty_print(),
         Expression::UnaryOperator { .. }
         | Expression::BinaryOperator { .. }
+        | Expression::BinaryOperatorForDate { .. }
         | Expression::Condition(..) => m::operator("(") + expr.pretty_print() + m::operator(")"),
     }
 }
@@ -542,6 +607,7 @@ impl PrettyPrint for Expression {
                 m::operator("!") + with_parens(expr)
             }
             BinaryOperator(_, op, lhs, rhs, _type) => pretty_print_binop(op, lhs, rhs),
+            BinaryOperatorForDate(_, op, lhs, rhs, _type) => pretty_print_binop(op, lhs, rhs),
             FunctionCall(_, _, name, args, _type) => {
                 m::identifier(name)
                     + m::operator("(")
@@ -552,6 +618,16 @@ impl PrettyPrint for Expression {
                     .sum()
                     + m::operator(")")
             }
+            CallableCall(_, expr, args, _type) => {
+                expr.pretty_print()
+                    + m::operator("(")
+                    + itertools::Itertools::intersperse(
+                        args.iter().map(|e| e.pretty_print()),
+                        m::operator(",") + m::space(),
+                    )
+                    .sum()
+                    + m::operator(")")
+            }
             Boolean(_, val) => val.pretty_print(),
             String(_, parts) => parts.pretty_print(),
             Condition(_, condition, then, else_) => {
@@ -608,7 +684,7 @@ mod tests {
 
                  @aliases(rad: short)
                  @metric_prefixes
-                 unit radian: Scalar
+                 unit radian: Scalar = meter / meter
 
                  @aliases(°: none)
                  unit degree = 180/pi × radian
@@ -616,6 +692,9 @@ mod tests {
                  @aliases(in: short)
                  unit inch = 0.0254 m
 
+                 @metric_prefixes
+                 unit points
+
                  let a = 1
                  let b = 1
                  let c = 1
@@ -731,6 +810,7 @@ mod tests {
         roundtrip_check("2^3!");
         roundtrip_check("-3!");
         roundtrip_check("(-3)!");
+        roundtrip_check("megapoints");
     }
 
     #[test]
