@@ -5,11 +5,12 @@ use crate::{
     interpreter::{InterpreterResult, PrintFunction, Result, RuntimeError},
     markup::Markup,
     math,
+    number::Number,
     prefix::Prefix,
     quantity::{Quantity, QuantityError},
     unit::Unit,
     unit_registry::{UnitMetadata, UnitRegistry},
-    value::Value,
+    value::{FunctionReference, Value},
 };
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
@@ -69,6 +70,13 @@ pub enum Op {
     LogicalOr,
     LogicalNeg,
 
+    /// Similar to Add, but has DateTime on the LHS and a quantity on the RHS
+    AddToDateTime,
+    /// Similar to Sub, but has DateTime on the LHS and a quantity on the RHS
+    SubFromDateTime,
+    /// Computes the difference between two DateTimes
+    DiffDateTime,
+
     /// Move IP forward by the given offset argument if the popped-of value on
     /// top of the stack is false.
     JumpIfFalse,
@@ -82,6 +90,9 @@ pub enum Op {
     /// Same as above, but call a procedure which does not return anything (does not push a value onto the stack)
     FFICallProcedure,
 
+    /// Call a callable object
+    CallCallable,
+
     /// Print a compile-time string
     PrintString,
 
@@ -106,11 +117,15 @@ impl Op {
             | Op::PrintString
             | Op::JoinString
             | Op::JumpIfFalse
-            | Op::Jump => 1,
+            | Op::Jump
+            | Op::CallCallable => 1,
             Op::Negate
             | Op::Factorial
             | Op::Add
+            | Op::AddToDateTime
             | Op::Subtract
+            | Op::SubFromDateTime
+            | Op::DiffDateTime
             | Op::Multiply
             | Op::Divide
             | Op::Power
@@ -141,7 +156,10 @@ impl Op {
             Op::Negate => "Negate",
             Op::Factorial => "Factorial",
             Op::Add => "Add",
+            Op::AddToDateTime => "AddDateTime",
             Op::Subtract => "Subtract",
+            Op::SubFromDateTime => "SubDateTime",
+            Op::DiffDateTime => "DiffDateTime",
             Op::Multiply => "Multiply",
             Op::Divide => "Divide",
             Op::Power => "Power",
@@ -160,6 +178,7 @@ impl Op {
             Op::Call => "Call",
             Op::FFICallFunction => "FFICallFunction",
             Op::FFICallProcedure => "FFICallProcedure",
+            Op::CallCallable => "CallCallable",
             Op::PrintString => "PrintString",
             Op::JoinString => "JoinString",
             Op::FullSimplify => "FullSimplify",
@@ -168,12 +187,13 @@ impl Op {
     }
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub enum Constant {
     Scalar(f64),
     Unit(Unit),
     Boolean(bool),
     String(String),
+    FunctionReference(FunctionReference),
 }
 
 impl Constant {
@@ -183,6 +203,7 @@ impl Constant {
             Constant::Unit(u) => Value::Quantity(Quantity::from_unit(u.clone())),
             Constant::Boolean(b) => Value::Boolean(*b),
             Constant::String(s) => Value::String(s.clone()),
+            Constant::FunctionReference(inner) => Value::FunctionReference(inner.clone()),
         }
     }
 }
@@ -194,6 +215,7 @@ impl Display for Constant {
             Constant::Unit(unit) => write!(f, "{}", unit),
             Constant::Boolean(val) => write!(f, "{}", val),
             Constant::String(val) => write!(f, "\"{}\"", val),
+            Constant::FunctionReference(inner) => write!(f, "{}", inner),
         }
     }
 }
@@ -495,6 +517,7 @@ impl Vm {
         self.stack.push(value);
     }
 
+    #[track_caller]
     fn pop_quantity(&mut self) -> Quantity {
         match self.pop() {
             Value::Quantity(q) => q,
@@ -502,10 +525,20 @@ impl Vm {
         }
     }
 
+    #[track_caller]
     fn pop_bool(&mut self) -> bool {
         self.pop().unsafe_as_bool()
     }
 
+    #[track_caller]
+    fn pop_datetime(&mut self) -> chrono::DateTime<chrono::FixedOffset> {
+        match self.pop() {
+            Value::DateTime(q) => q,
+            _ => panic!("Expected datetime to be on the top of the stack"),
+        }
+    }
+
+    #[track_caller]
     fn pop(&mut self) -> Value {
         self.stack.pop().expect("stack should not be empty")
     }
@@ -577,7 +610,7 @@ impl Vm {
 
                     self.constants[constant_idx as usize] = Constant::Unit(Unit::new_derived(
                         &unit_information.0,
-                        unit_information.1.as_ref().unwrap(),
+                        unit_information.2.canonical_name.clone(),
                         *conversion_value.unsafe_value(),
                         defining_unit.clone(),
                     ));
@@ -615,6 +648,46 @@ impl Vm {
                     };
                     self.push_quantity(result.map_err(RuntimeError::QuantityError)?);
                 }
+                op @ (Op::AddToDateTime | Op::SubFromDateTime) => {
+                    let rhs = self.pop_quantity();
+                    let lhs = self.pop_datetime();
+
+                    // for time, the base unit is in seconds
+                    let base = rhs.to_base_unit_representation();
+                    let seconds_f = base.unsafe_value().to_f64();
+
+                    let duration = chrono::Duration::try_seconds(seconds_f.trunc() as i64)
+                        .ok_or(RuntimeError::DurationOutOfRange)?
+                        + chrono::Duration::nanoseconds(
+                            (seconds_f.fract() * 1_000_000_000f64).round() as i64,
+                        );
+
+                    self.push(Value::DateTime(match op {
+                        Op::AddToDateTime => lhs
+                            .checked_add_signed(duration)
+                            .ok_or(RuntimeError::DateTimeOutOfRange)?,
+                        Op::SubFromDateTime => lhs
+                            .checked_sub_signed(duration)
+                            .ok_or(RuntimeError::DateTimeOutOfRange)?,
+                        _ => unreachable!(),
+                    }));
+                }
+                Op::DiffDateTime => {
+                    let unit = self.pop_quantity();
+                    let rhs = self.pop_datetime();
+                    let lhs = self.pop_datetime();
+
+                    let duration = lhs - rhs;
+                    let duration = duration.subsec_nanos() as f64 / 1_000_000_000f64
+                        + duration.num_seconds() as f64;
+
+                    let ret = Value::Quantity(Quantity::new(
+                        Number::from_f64(duration),
+                        unit.unit().clone(),
+                    ));
+
+                    self.push(ret);
+                }
                 op @ (Op::LessThan | Op::GreaterThan | Op::LessOrEqual | Op::GreatorOrEqual) => {
                     let rhs = self.pop_quantity();
                     let lhs = self.pop_quantity();
@@ -730,6 +803,56 @@ impl Vm {
                         }
                     }
                 }
+                Op::CallCallable => {
+                    let num_args = self.read_u16() as usize;
+
+                    let callable = self.pop();
+                    match callable.unsafe_as_function_reference() {
+                        FunctionReference::Normal(name) => {
+                            let function_idx = self.get_function_idx(name) as usize;
+
+                            // TODO: unify code with 'Op::Call'?
+                            self.frames.push(CallFrame {
+                                function_idx,
+                                ip: 0,
+                                fp: self.stack.len() - num_args,
+                            })
+                        }
+                        FunctionReference::Foreign(name) => {
+                            let function_idx = self
+                                .get_ffi_callable_idx(name)
+                                .expect("Foreign function exists")
+                                as usize;
+
+                            let mut args = vec![];
+                            for _ in 0..num_args {
+                                args.push(self.pop());
+                            }
+                            args.reverse();
+
+                            match &self.ffi_callables[function_idx].callable {
+                                Callable::Function(function) => {
+                                    let result = (function)(&args[..]);
+                                    self.push(result?);
+                                }
+                                Callable::Procedure(..) => unreachable!("Foreign procedures can not be targeted by a function reference"),
+                            }
+                        }
+                        FunctionReference::TzConversion(tz_name) => {
+                            // TODO: implement this using a closure, once we have that in the language
+
+                            let dt = self.pop_datetime();
+
+                            let tz: chrono_tz::Tz = tz_name
+                                .parse()
+                                .map_err(|_| RuntimeError::UnknownTimezone(tz_name.into()))?;
+
+                            let dt = dt.with_timezone(&tz).fixed_offset();
+
+                            self.push(Value::DateTime(dt));
+                        }
+                    }
+                }
                 Op::PrintString => {
                     let s_idx = self.read_u16() as usize;
                     let s = &self.strings[s_idx];
@@ -743,6 +866,8 @@ impl Vm {
                             Value::Quantity(q) => q.to_string(),
                             Value::Boolean(b) => b.to_string(),
                             Value::String(s) => s,
+                            Value::DateTime(dt) => crate::datetime::to_rfc2822_save(&dt),
+                            Value::FunctionReference(r) => r.to_string(),
                         };
                         joined = part + &joined; // reverse order
                     }
