@@ -342,14 +342,14 @@ fn test_incompatible_dimension_errors() {
 fn test_temperature_conversions() {
     expect_output("from_celsius(11.5)", "284.65 K");
     expect_output("from_fahrenheit(89.3)", "304.983 K");
-    expect_output("to_celsius(0 K)", "-273.15");
-    expect_output("to_fahrenheit(30 K)", "-405.67");
-    expect_output("to_celsius(from_celsius(100))", "100");
-    expect_output("to_fahrenheit(from_fahrenheit(100))", "100.0");
-    expect_output("from_celsius(to_celsius(123 K))", "123 K");
-    expect_output("from_fahrenheit(to_fahrenheit(123 K))", "123 K");
+    expect_output("0 K -> celsius", "-273.15");
+    expect_output("fahrenheit(30 K)", "-405.67");
+    expect_output("from_celsius(100) -> celsius", "100");
+    expect_output("from_fahrenheit(100) -> fahrenheit", "100.0");
+    expect_output("from_celsius(123 K -> celsius)", "123 K");
+    expect_output("from_fahrenheit(123 K -> fahrenheit)", "123 K");
 
-    expect_output("-40 // from_fahrenheit // to_celsius", "-40");
+    expect_output("-40 -> from_fahrenheit -> celsius", "-40");
 }
 
 #[test]
@@ -590,3 +590,31 @@ fn test_overwrite_captured_constant() {
         "0.841471",
     );
 }
+
+#[test]
+fn test_pretty_print_prefixes() {
+    expect_output("1 megabarn", "1 megabarn");
+}
+
+#[test]
+fn test_full_simplify_for_function_calls() {
+    expect_output("floor(1.2 hours / hour)", "1");
+}
+
+#[test]
+fn test_datetime_runtime_errors() {
+    expect_failure("datetime(\"2000-01-99\")", "Unrecognized datetime format");
+    expect_failure("now() -> tz(\"Europe/NonExisting\")", "Unknown timezone");
+    expect_failure(
+        "date(\"2000-01-01\") + 1e100 years",
+        "Exceeded maximum size for time durations",
+    );
+    expect_failure(
+        "date(\"2000-01-01\") + 100000000 years",
+        "DateTime out of range",
+    );
+    expect_failure(
+        "format_datetime(\"%Y-%m-%dT%H%:M\", now())",
+        "Error in datetime format",
+    )
+}
